@@ -1,24 +1,115 @@
-# Copyright 2008-2017 pydicom authors. See LICENSE file for details.
+# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
 """Functions related to writing DICOM data."""
+
 from __future__ import absolute_import
 from struct import pack
 
 from pydicom import compat
 from pydicom.compat import in_py2
-from pydicom.charset import default_encoding, text_VRs, convert_encodings
-from pydicom.datadict import keyword_for_tag
+from pydicom.charset import (
+    default_encoding, text_VRs, convert_encodings, encode_string
+)
 from pydicom.dataelem import DataElement_from_raw
-from pydicom.dataset import Dataset
+from pydicom.dataset import Dataset, validate_file_meta
 from pydicom.filebase import DicomFile, DicomFileLike, DicomBytesIO
 from pydicom.multival import MultiValue
 from pydicom.tag import (Tag, ItemTag, ItemDelimiterTag, SequenceDelimiterTag,
                          tag_in_exception)
-from pydicom.uid import (PYDICOM_IMPLEMENTATION_UID, ImplicitVRLittleEndian,
-                         ExplicitVRBigEndian,
-                         UncompressedPixelTransferSyntaxes)
-from pydicom.valuerep import extra_length_VRs
+from pydicom.uid import UncompressedPixelTransferSyntaxes
+from pydicom.valuerep import extra_length_VRs, PersonNameUnicode
 from pydicom.values import convert_numbers
-from pydicom._version import __version_info__
+
+
+def _correct_ambiguous_vr_element(elem, ds, is_little_endian):
+    """Implementation for `correct_ambiguous_vr_element`.
+    See `correct_ambiguous_vr_element` for description.
+    """
+    # 'OB or OW': 7fe0,0010 PixelData
+    if elem.tag == 0x7fe00010:
+        # Compressed Pixel Data
+        # PS3.5 Annex A.4
+        #   If encapsulated, VR is OB and length is undefined
+        if elem.is_undefined_length:
+            elem.VR = 'OB'
+        # Non-compressed Pixel Data - Implicit Little Endian
+        # PS3.5 Annex A1: VR is always OW
+        elif ds.is_implicit_VR:
+            elem.VR = 'OW'
+        else:
+            # Non-compressed Pixel Data - Explicit VR
+            # PS3.5 Annex A.2:
+            # If BitsAllocated is > 8 then VR shall be OW,
+            # else may be OB or OW.
+            # If we get here, the data has not been written before
+            # or has been converted from Implicit Little Endian,
+            # so we default to OB for BitsAllocated 1 or 8
+            elem.VR = 'OW' if ds.BitsAllocated > 8 else 'OB'
+
+    # 'US or SS' and dependent on PixelRepresentation
+    # (0018,9810) Zero Velocity Pixel Value
+    # (0022,1452) Mapped Pixel Value
+    # (0028,0104)/(0028,0105) Smallest/Largest Valid Pixel Value
+    # (0028,0106)/(0028,0107) Smallest/Largest Image Pixel Value
+    # (0028,0108)/(0028,0109) Smallest/Largest Pixel Value in Series
+    # (0028,0110)/(0028,0111) Smallest/Largest Image Pixel Value in Plane
+    # (0028,0120) Pixel Padding Value
+    # (0028,0121) Pixel Padding Range Limit
+    # (0028,1101-1103) Red/Green/Blue Palette Color Lookup Table Descriptor
+    # (0028,3002) LUT Descriptor
+    # (0040,9216)/(0040,9211) Real World Value First/Last Value Mapped
+    # (0060,3004)/(0060,3006) Histogram First/Last Bin Value
+    elif elem.tag in [
+            0x00189810, 0x00221452, 0x00280104, 0x00280105, 0x00280106,
+            0x00280107, 0x00280108, 0x00280109, 0x00280110, 0x00280111,
+            0x00280120, 0x00280121, 0x00281101, 0x00281102, 0x00281103,
+            0x00283002, 0x00409211, 0x00409216, 0x00603004, 0x00603006
+    ]:
+        # US if PixelRepresentation value is 0x0000, else SS
+        #   For references, see the list at
+        #   https://github.com/darcymason/pydicom/pull/298
+        if ds.PixelRepresentation == 0:
+            elem.VR = 'US'
+            byte_type = 'H'
+        else:
+            elem.VR = 'SS'
+            byte_type = 'h'
+        elem.value = convert_numbers(elem.value, is_little_endian,
+                                     byte_type)
+
+    # 'OB or OW' and dependent on WaveformBitsAllocated
+    # (5400, 0110) Channel Minimum Value
+    # (5400, 0112) Channel Maximum Value
+    # (5400, 100A) Waveform Padding Data
+    # (5400, 1010) Waveform Data
+    elif elem.tag in [0x54000110, 0x54000112, 0x5400100A, 0x54001010]:
+        # If WaveformBitsAllocated is > 8 then OW, otherwise may be
+        #   OB or OW.
+        #   See PS3.3 C.10.9.1.
+        if ds.is_implicit_VR:
+            elem.VR = 'OW'
+        else:
+            elem.VR = 'OW' if ds.WaveformBitsAllocated > 8 else 'OB'
+
+    # 'US or OW': 0028,3006 LUTData
+    elif elem.tag == 0x00283006:
+        # First value in LUT Descriptor is how many values in
+        #   LUTData, if there's only one value then must be US
+        # As per PS3.3 C.11.1.1.1
+        if ds.LUTDescriptor[0] == 1:
+            elem.VR = 'US'
+            elem.value = convert_numbers(elem.value, is_little_endian,
+                                         'H')
+        else:
+            elem.VR = 'OW'
+
+    # 'OB or OW': 60xx,3000 OverlayData and dependent on Transfer Syntax
+    elif (elem.tag.group in range(0x6000, 0x601F, 2)
+          and elem.tag.elem == 0x3000):
+        # Implicit VR must be OW, explicit VR may be OB or OW
+        #   as per PS3.5 Section 8.1.2 and Annex A
+        elem.VR = 'OW'
+
+    return elem
 
 
 def correct_ambiguous_vr_element(elem, ds, is_little_endian):
@@ -51,83 +142,12 @@ def correct_ambiguous_vr_element(elem, ds, is_little_endian):
             elem = DataElement_from_raw(elem)
             ds.__setitem__(elem.tag, elem)
 
-        # 'OB or OW': 7fe0,0010 PixelData
-        if elem.tag == 0x7fe00010:
-
-            try:
-                # Compressed Pixel Data
-                # PS3.5 Annex A.4
-                #   If encapsulated, VR is OB and length is undefined
-                if elem.is_undefined_length:
-                    elem.VR = 'OB'
-                else:
-                    # Non-compressed Pixel Data
-                    # If BitsAllocated is > 8 then OW, else may be OB or OW
-                    #   as per PS3.5 Annex A.2. For BitsAllocated < 8 test the
-                    #    size of each pixel to see if its written in OW or OB
-                    if ds.BitsAllocated > 8:
-                        elem.VR = 'OW'
-                    else:
-                        nr_pixels = ds.Rows * ds.Columns
-                        if 'SamplesPerPixel' in ds:
-                            nr_pixels *= ds.SamplesPerPixel
-                        pixel_size = len(ds.PixelData) / nr_pixels
-                        if pixel_size == 2:
-                            elem.VR = 'OW'
-                        elif pixel_size == 1:
-                            elem.VR = 'OB'
-            except AttributeError:
-                pass
-
-        # 'US or SS' and dependent on PixelRepresentation
-        elif elem.tag in [
-                0x00189810, 0x00221452, 0x00280104, 0x00280105, 0x00280106,
-                0x00280107, 0x00280108, 0x00280109, 0x00280110, 0x00280111,
-                0x00280120, 0x00280121, 0x00281101, 0x00281102, 0x00281103,
-                0x00283002, 0x00409211, 0x00409216, 0x00603004, 0x00603006
-        ]:
-            # US if PixelRepresenation value is 0x0000, else SS
-            #   For references, see the list at
-            #   https://github.com/darcymason/pydicom/pull/298
-            if 'PixelRepresentation' in ds:
-                if ds.PixelRepresentation == 0:
-                    elem.VR = 'US'
-                    byte_type = 'H'
-                else:
-                    elem.VR = 'SS'
-                    byte_type = 'h'
-                elem.value = convert_numbers(elem.value, is_little_endian,
-                                             byte_type)
-
-        # 'OB or OW' and dependent on WaveformBitsAllocated
-        elif elem.tag in [0x54000100, 0x54000112, 0x5400100A, 0x54001010]:
-            # If WaveformBitsAllocated is > 8 then OW, otherwise may be
-            #   OB or OW, however not sure how to handle this.
-            #   See PS3.3 C.10.9.1.
-            if 'WaveformBitsAllocated' in ds:
-                if ds.WaveformBitsAllocated > 8:
-                    elem.VR = 'OW'
-
-        # 'US or OW': 0028,3006 LUTData
-        elif elem.tag in [0x00283006]:
-            if 'LUTDescriptor' in ds:
-                # First value in LUT Descriptor is how many values in
-                #   LUTData, if there's only one value then must be US
-                # As per PS3.3 C.11.1.1.1
-                if ds.LUTDescriptor[0] == 1:
-                    elem.VR = 'US'
-                    elem.value = convert_numbers(elem.value, is_little_endian,
-                                                 'H')
-                else:
-                    elem.VR = 'OW'
-
-        # 'OB or OW': 60xx,3000 OverlayData and dependent on Transfer Syntax
-        elif (elem.tag.group in range(0x6000, 0x601F, 2)
-              and elem.tag.elem == 0x3000):
-            # Implicit VR must be OW, explicit VR may be OB or OW
-            #   as per PS3.5 Section 8.1.2 and Annex A
-            if ds.is_implicit_VR:
-                elem.VR = 'OW'
+        try:
+            _correct_ambiguous_vr_element(elem, ds, is_little_endian)
+        except AttributeError as e:
+            reason = ('Failed to resolve ambiguous VR for tag'
+                      ' {}: '.format(elem.tag)) + str(e)
+            raise AttributeError(reason)
 
     return elem
 
@@ -153,6 +173,11 @@ def correct_ambiguous_vr(ds, is_little_endian):
     -------
     ds : pydicom.dataset.Dataset
         The corrected dataset
+
+    Raises
+    ------
+    AttributeError
+        If a tag is missing in `ds` that is required to resolve the ambiguity.
     """
     # Iterate through the elements
     for elem in ds:
@@ -220,15 +245,14 @@ def _is_multi_value(val):
 def multi_string(val):
     """Put a string together with delimiter if has more than one value"""
     if _is_multi_value(val):
-        # \ is escape chr, so "\\" gives single backslash
         return "\\".join(val)
     else:
         return val
 
 
-def write_PN(fp, data_element, padding=b' ', encoding=None):
-    if not encoding:
-        encoding = [default_encoding] * 3
+def write_PN(fp, data_element, encodings=None):
+    if not encodings:
+        encodings = [default_encoding]
 
     if data_element.VM == 1:
         val = [data_element.value, ]
@@ -237,30 +261,52 @@ def write_PN(fp, data_element, padding=b' ', encoding=None):
 
     if isinstance(val[0], compat.text_type) or not in_py2:
         try:
-            val = [elem.encode(encoding) for elem in val]
+            val = [elem.encode(encodings) for elem in val]
         except TypeError:
-            val = [elem.encode(encoding[0]) for elem in val]
+            # we get here in Python 2 if val is a unicode string
+            val = [PersonNameUnicode(elem, encodings) for elem in val]
+            val = [elem.encode(encodings) for elem in val]
 
     val = b'\\'.join(val)
 
     if len(val) % 2 != 0:
-        val = val + padding
+        val = val + b' '
 
     fp.write(val)
 
 
-def write_string(fp, data_element, padding=' ', encoding=default_encoding):
-    """Write a single or multivalued string."""
+def write_string(fp, data_element, padding=' '):
+    """Write a single or multivalued ASCII string."""
     val = multi_string(data_element.value)
     if val is not None:
         if len(val) % 2 != 0:
             val = val + padding  # pad to even length
         if isinstance(val, compat.text_type):
-            val = val.encode(encoding)
+            val = val.encode(default_encoding)
         fp.write(val)
 
 
-def write_number_string(fp, data_element, padding=' '):
+def write_text(fp, data_element, encodings=None):
+    """Write a single or multivalued text string."""
+    val = data_element.value
+    if val is not None:
+        encodings = encodings or [default_encoding]
+        if _is_multi_value(val):
+            if val and isinstance(val[0], compat.text_type):
+                val = b'\\'.join([encode_string(val, encodings)
+                                  for val in val])
+            else:
+                val = b'\\'.join([val for val in val])
+        else:
+            if isinstance(val, compat.text_type):
+                val = encode_string(val, encodings)
+
+        if len(val) % 2 != 0:
+            val = val + b' '  # pad to even length
+        fp.write(val)
+
+
+def write_number_string(fp, data_element):
     """Handle IS or DS VR - write a number stored as a string of digits."""
     # If the DS or IS has an original_string attribute, use that, so that
     # unchanged data elements are written with exact string as when read from
@@ -278,7 +324,7 @@ def write_number_string(fp, data_element, padding=' '):
             val = str(val)
 
     if len(val) % 2 != 0:
-        val = val + padding  # pad to even length
+        val = val + ' '  # pad to even length
 
     if not in_py2:
         val = bytes(val, default_encoding)
@@ -295,10 +341,10 @@ def _format_DA(val):
         return val.strftime("%Y%m%d")
 
 
-def write_DA(fp, data_element, padding=' '):
+def write_DA(fp, data_element):
     val = data_element.value
     if isinstance(val, (str, compat.string_types)):
-        write_string(fp, data_element, padding)
+        write_string(fp, data_element)
     else:
         if _is_multi_value(val):
             val = "\\".join((x if isinstance(x, (str, compat.string_types))
@@ -306,7 +352,7 @@ def write_DA(fp, data_element, padding=' '):
         else:
             val = _format_DA(val)
         if len(val) % 2 != 0:
-            val = val + padding  # pad to even length
+            val = val + ' '  # pad to even length
 
         if isinstance(val, compat.string_types):
             val = val.encode(default_encoding)
@@ -323,10 +369,10 @@ def _format_DT(val):
         return val.strftime("%Y%m%d%H%M%S%z")
 
 
-def write_DT(fp, data_element, padding=' '):
+def write_DT(fp, data_element):
     val = data_element.value
     if isinstance(val, (str, compat.string_types)):
-        write_string(fp, data_element, padding)
+        write_string(fp, data_element)
     else:
         if _is_multi_value(val):
             val = "\\".join((x if isinstance(x, (str, compat.string_types))
@@ -334,7 +380,7 @@ def write_DT(fp, data_element, padding=' '):
         else:
             val = _format_DT(val)
         if len(val) % 2 != 0:
-            val = val + padding  # pad to even length
+            val = val + ' '  # pad to even length
 
         if isinstance(val, compat.string_types):
             val = val.encode(default_encoding)
@@ -353,10 +399,10 @@ def _format_TM(val):
         return val.strftime("%H%M%S")
 
 
-def write_TM(fp, data_element, padding=' '):
+def write_TM(fp, data_element):
     val = data_element.value
     if isinstance(val, (str, compat.string_types)):
-        write_string(fp, data_element, padding)
+        write_string(fp, data_element)
     else:
         if _is_multi_value(val):
             val = "\\".join((x if isinstance(x, (str, compat.string_types))
@@ -364,7 +410,7 @@ def write_TM(fp, data_element, padding=' '):
         else:
             val = _format_TM(val)
         if len(val) % 2 != 0:
-            val = val + padding  # pad to even length
+            val = val + ' '  # pad to even length
 
         if isinstance(val, compat.string_types):
             val = val.encode(default_encoding)
@@ -372,8 +418,9 @@ def write_TM(fp, data_element, padding=' '):
         fp.write(val)
 
 
-def write_data_element(fp, data_element, encoding=default_encoding):
-    """Write the data_element to file fp according to dicom media storage rules.
+def write_data_element(fp, data_element, encodings=None):
+    """Write the data_element to file fp according to
+    dicom media storage rules.
     """
     # Write element's tag
     fp.write_tag(data_element.tag)
@@ -409,13 +456,12 @@ def write_data_element(fp, data_element, encoding=default_encoding):
                 "write_data_element: unknown Value Representation "
                 "'{0}'".format(VR))
 
-        encoding = convert_encodings(encoding)
+        encodings = encodings or [default_encoding]
+        encodings = convert_encodings(encodings)
         writer_function, writer_param = writers[VR]
         is_undefined_length = data_element.is_undefined_length
-        if VR in text_VRs:
-            writer_function(buffer, data_element, encoding=encoding[1])
-        elif VR in ('PN', 'SQ'):
-            writer_function(buffer, data_element, encoding=encoding)
+        if VR in text_VRs or VR in ('PN', 'SQ'):
+            writer_function(buffer, data_element, encodings=encodings)
         else:
             # Many numeric types use the same writer but with numeric format
             # parameter
@@ -493,16 +539,16 @@ def _harmonize_properties(dataset, fp):
     fp.is_little_endian = dataset.is_little_endian
 
 
-def write_sequence(fp, data_element, encoding):
+def write_sequence(fp, data_element, encodings):
     """Write a dicom Sequence contained in data_element to the file fp."""
     # write_data_element has already written the VR='SQ' (if needed) and
     #    a placeholder for length"""
     sequence = data_element.value
     for dataset in sequence:
-        write_sequence_item(fp, dataset, encoding)
+        write_sequence_item(fp, dataset, encodings)
 
 
-def write_sequence_item(fp, dataset, encoding):
+def write_sequence_item(fp, dataset, encodings):
     """Write an item (dataset) in a dicom Sequence to the dicom file fp.
 
     This is similar to writing a data_element, but with a specific tag for
@@ -514,7 +560,7 @@ def write_sequence_item(fp, dataset, encoding):
     length_location = fp.tell()  # save location for later.
     # will fill in real value later if not undefined length
     fp.write_UL(0xffffffff)
-    write_dataset(fp, dataset, parent_encoding=encoding)
+    write_dataset(fp, dataset, parent_encoding=encodings)
     if getattr(dataset, "is_undefined_length_sequence_item", False):
         fp.write_tag(ItemDelimiterTag)
         fp.write_UL(0)  # 4-bytes 'length' field for delimiter item
@@ -606,41 +652,11 @@ def write_file_meta_info(fp, file_meta, enforce_standard=True):
     ValueError
         If any non-Group 2 Elements are present in `file_meta`.
     """
-    # Check that no non-Group 2 Elements are present
-    for elem in file_meta.elements():
-        if elem.tag.group != 0x0002:
-            raise ValueError("Only File Meta Information Group (0002,eeee) "
-                             "elements must be present in 'file_meta'.")
+    validate_file_meta(file_meta, enforce_standard)
 
-    # The Type 1 File Meta Elements are only required when `enforce_standard`
-    #   is True, except for FileMetaInformationGroupLength and
-    #   FileMetaInformationVersion, which may or may not be present
-    if enforce_standard:
+    if enforce_standard and 'FileMetaInformationGroupLength' not in file_meta:
         # Will be updated with the actual length later
-        if 'FileMetaInformationGroupLength' not in file_meta:
-            file_meta.FileMetaInformationGroupLength = 0
-
-        if 'FileMetaInformationVersion' not in file_meta:
-            file_meta.FileMetaInformationVersion = b'\x00\x01'
-
-        if 'ImplementationClassUID' not in file_meta:
-            file_meta.ImplementationClassUID = PYDICOM_IMPLEMENTATION_UID
-
-        if 'ImplementationVersionName' not in file_meta:
-            file_meta.ImplementationVersionName = (
-                'PYDICOM ' + ".".join(str(x) for x in __version_info__))
-
-        # Check that required File Meta Elements are present
-        missing = []
-        for element in [0x0002, 0x0003, 0x0010]:
-            if Tag(0x0002, element) not in file_meta:
-                missing.append(Tag(0x0002, element))
-        if missing:
-            msg = "Missing required File Meta Information elements from " \
-                  "'file_meta':\n"
-            for tag in missing:
-                msg += '\t{0} {1}\n'.format(tag, keyword_for_tag(tag))
-            raise ValueError(msg[:-1])  # Remove final newline
+        file_meta.FileMetaInformationGroupLength = 0
 
     # Only used if FileMetaInformationGroupLength is present.
     #   FileMetaInformationGroupLength has a VR of 'UL' and so has a value that
@@ -803,28 +819,11 @@ def dcmwrite(filename, dataset, write_like_original=True):
 
     # File Meta Information is required under the DICOM standard, however if
     #   `write_like_original` is True we treat it as optional
-    file_meta = getattr(dataset, 'file_meta', None)
-    if not file_meta and not write_like_original:
-        dataset.file_meta = Dataset()
-        file_meta = dataset.file_meta
-
-    # If enforcing the standard, correct the TransferSyntaxUID where possible,
-    #   noting that the transfer syntax for is_implicit_VR = False and
-    #   is_little_endian = True is ambiguous as it may be an encapsulated
-    #   transfer syntax
     if not write_like_original:
-        if dataset.is_little_endian and dataset.is_implicit_VR:
-            file_meta.TransferSyntaxUID = ImplicitVRLittleEndian
-        elif not dataset.is_little_endian and not dataset.is_implicit_VR:
-            file_meta.TransferSyntaxUID = ExplicitVRBigEndian
-        elif not dataset.is_little_endian and dataset.is_implicit_VR:
-            raise NotImplementedError("Implicit VR Big Endian is not a "
-                                      "supported Transfer Syntax.")
-
-        if 'SOPClassUID' in dataset:
-            file_meta.MediaStorageSOPClassUID = dataset.SOPClassUID
-        if 'SOPInstanceUID' in dataset:
-            file_meta.MediaStorageSOPInstanceUID = dataset.SOPInstanceUID
+        # the checks will be done in write_file_meta_info()
+        dataset.fix_meta_info(enforce_standard=False)
+    else:
+        dataset.ensure_file_meta()
 
     # Check for decompression, give warnings if inconsistencies
     # If decompressed, then pixel_array is now used instead of PixelData
@@ -862,9 +861,9 @@ def dcmwrite(filename, dataset, write_like_original=True):
             fp.write(preamble)
             fp.write(b'DICM')
 
-        if file_meta is not None:  # May be an empty Dataset
+        if dataset.file_meta:  # May be an empty Dataset
             # If we want to `write_like_original`, don't enforce_standard
-            write_file_meta_info(fp, file_meta,
+            write_file_meta_info(fp, dataset.file_meta,
                                  enforce_standard=not write_like_original)
 
         # WRITE DATASET
@@ -909,23 +908,23 @@ writers = {
     'OD': (write_OWvalue, None),
     'OL': (write_OWvalue, None),
     'UI': (write_UI, None),
-    'SH': (write_string, None),
+    'SH': (write_text, None),
     'DA': (write_DA, None),
     'TM': (write_TM, None),
     'CS': (write_string, None),
     'PN': (write_PN, None),
-    'LO': (write_string, None),
+    'LO': (write_text, None),
     'IS': (write_number_string, None),
     'DS': (write_number_string, None),
     'AE': (write_string, None),
     'AS': (write_string, None),
-    'LT': (write_string, None),
+    'LT': (write_text, None),
     'SQ': (write_sequence, None),
-    'UC': (write_string, None),
+    'UC': (write_text, None),
     'UN': (write_UN, None),
     'UR': (write_string, None),
     'AT': (write_ATvalue, None),
-    'ST': (write_string, None),
+    'ST': (write_text, None),
     'OW': (write_OWvalue, None),
     'US or SS': (write_OWvalue, None),
     'US or OW': (write_OWvalue, None),
@@ -935,5 +934,5 @@ writers = {
     'OB or OW': (write_OBvalue, None),
     'OW or OB': (write_OBvalue, None),
     'DT': (write_DT, None),
-    'UT': (write_string, None),
+    'UT': (write_text, None),
 }  # note OW/OB depends on other items, which we don't know at write time
