@@ -1,9 +1,6 @@
+# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
 # -*- coding: utf-8 -*-
 """unittest tests for pydicom.filereader module"""
-# Copyright (c) 2010-2012 Darcy Mason
-# This file is part of pydicom, released under a modified MIT license.
-#    See the file LICENSE included with this distribution, also
-#    available at https://github.com/pydicom/pydicom
 
 import gzip
 from io import BytesIO
@@ -303,8 +300,10 @@ class ReaderTests(unittest.TestCase):
             Tag(0x0010, 0x0010), 'PatientID', 'ImageType', 'ViewName'])
         ctspecific_tags = sorted(ctspecific.keys())
         expected = [
+            # SpecificCharacterSet is always added
             # ViewName does not exist in the data set
-            Tag(0x0008, 0x0008), Tag(0x0010, 0x0010), Tag(0x0010, 0x0020)
+            Tag(0x0008, 0x0005), Tag(0x0008, 0x0008),
+            Tag(0x0010, 0x0010), Tag(0x0010, 0x0020)
         ]
         self.assertEqual(expected, ctspecific_tags)
 
@@ -327,7 +326,8 @@ class ReaderTests(unittest.TestCase):
         tags = dcmread(emri_jpeg_2k_lossless, specific_tags=[
             unknown_len_tag])
         tags = sorted(tags.keys())
-        self.assertEqual([unknown_len_tag], tags)
+        # SpecificCharacterSet is always added
+        self.assertEqual([Tag(0x08, 0x05), unknown_len_tag], tags)
 
         tags = dcmread(emri_jpeg_2k_lossless, specific_tags=[
             'SpecificCharacterSet'])
@@ -886,12 +886,13 @@ class ReadTruncatedFileTests(unittest.TestCase):
     def testReadFileWithMissingPixelDataArray(self):
         mr = dcmread(truncated_mr_name)
         mr.decode()
-        msg = (r"(Amount of pixel data.*"
-               "does not match the expected data|"
-               "Unexpected end of file. Read.*bytes of.*expected|"
-               "'str' object has no attribute 'reshape'|"
-               "'bytes' object has no attribute 'reshape')")
-        with pytest.raises(AttributeError, match=msg):
+        # Need to escape brackets
+        msg = (
+            r"The length of the pixel data in the dataset doesn't match the "
+            r"expected amount \(8130 vs. 8192 bytes\). The dataset may be "
+            r"corrupted or there may be an issue with the pixel data handler."
+        )
+        with pytest.raises(ValueError, match=msg):
             mr.pixel_array
 
 
