@@ -1,4 +1,6 @@
+# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
 # -*- coding: utf-8 -*-
+
 import unittest
 import os
 import sys
@@ -8,6 +10,7 @@ import pytest
 import pydicom
 from pydicom.filereader import dcmread
 from pydicom.data import get_testdata_files
+from pydicom.pixel_data_handlers.util import _convert_YBR_FULL_to_RGB
 from pydicom.tag import Tag
 from pydicom import compat
 gdcm_missing_message = "GDCM is not available in this test environment"
@@ -25,19 +28,20 @@ try:
 except AttributeError:
     have_pytest_param = False
 
-gdcm_handler = None
-have_gdcm_handler = True
 try:
     import pydicom.pixel_data_handlers.gdcm_handler as gdcm_handler
+    HAVE_GDCM = gdcm_handler.HAVE_GDCM
 except ImportError as e:
-    have_gdcm_handler = False
-numpy_handler = None
-have_numpy_handler = True
+    HAVE_GDCM = False
+    gdcm_handler = None
+
 try:
     import pydicom.pixel_data_handlers.numpy_handler as numpy_handler
+    HAVE_NP = numpy_handler.HAVE_NP
 except ImportError:
-    have_numpy_handler = False
-test_gdcm_decoder = have_gdcm_handler
+    HAVE_NP = False
+    numpy_handler = None
+
 
 empty_number_tags_name = get_testdata_files(
     "reportsi_with_empty_number_tags.dcm")[0]
@@ -114,11 +118,11 @@ class GDCM_JPEG_LS_Tests_no_gdcm(unittest.TestCase):
         self.mr_small = dcmread(mr_name)
         self.emri_jpeg_ls_lossless = dcmread(emri_jpeg_ls_lossless)
         self.emri_small = dcmread(emri_name)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [None]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = []
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
         os.remove(self.unicode_filename)
 
     def test_JPEG_LS_PixelArray(self):
@@ -138,11 +142,11 @@ class GDCM_JPEG2000Tests_no_gdcm(unittest.TestCase):
         self.emri_jpeg_2k_lossless = dcmread(emri_jpeg_2k_lossless)
         self.emri_small = dcmread(emri_name)
         self.sc_rgb_jpeg2k_gdcm_KY = dcmread(sc_rgb_jpeg2k_gdcm_KY)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [None]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = []
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
 
     def test_JPEG2000(self):
         """JPEG2000: Returns correct values for sample data elements"""
@@ -181,11 +185,11 @@ class GDCM_JPEGlossyTests_no_gdcm(unittest.TestCase):
     def setUp(self):
         self.jpeg_lossy = dcmread(jpeg_lossy_name)
         self.color_3d_jpeg = dcmread(color_3d_jpeg_baseline)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [None]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = []
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
 
     def test_JPEGlossy(self):
         """JPEG-lossy: Returns correct values for sample data elements"""
@@ -209,11 +213,11 @@ class GDCM_JPEGlossyTests_no_gdcm(unittest.TestCase):
 class GDCM_JPEGlosslessTests_no_gdcm(unittest.TestCase):
     def setUp(self):
         self.jpeg_lossless = dcmread(jpeg_lossless_name)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [None]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = []
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
 
     def testJPEGlossless(self):
         """JPEGlossless: Returns correct values for sample data elements"""
@@ -234,7 +238,7 @@ class GDCM_JPEGlosslessTests_no_gdcm(unittest.TestCase):
             _ = self.jpeg_lossless.pixel_array
 
 
-@pytest.mark.skipif(not test_gdcm_decoder, reason=gdcm_missing_message)
+@pytest.mark.skipif(not HAVE_GDCM, reason=gdcm_missing_message)
 class GDCM_JPEG_LS_Tests_with_gdcm(unittest.TestCase):
     def setUp(self):
         if compat.in_py2:
@@ -251,11 +255,11 @@ class GDCM_JPEG_LS_Tests_with_gdcm(unittest.TestCase):
         self.mr_small = dcmread(mr_name)
         self.emri_jpeg_ls_lossless = dcmread(emri_jpeg_ls_lossless)
         self.emri_small = dcmread(emri_name)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [numpy_handler, gdcm_handler]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
         os.remove(self.unicode_filename)
 
     def test_JPEG_LS_PixelArray(self):
@@ -267,6 +271,8 @@ class GDCM_JPEG_LS_Tests_with_gdcm(unittest.TestCase):
             "using GDCM Decoded pixel data is not "
             "all {0} (mean == {1})".format(b.mean(), a.mean()))
 
+        assert a.flags.writeable
+
     def test_emri_JPEG_LS_PixelArray_with_gdcm(self):
         a = self.emri_jpeg_ls_lossless.pixel_array
         b = self.emri_small.pixel_array
@@ -276,8 +282,10 @@ class GDCM_JPEG_LS_Tests_with_gdcm(unittest.TestCase):
             "Decoded pixel data is not all {0} "
             "(mean == {1})".format(b.mean(), a.mean()))
 
+        assert a.flags.writeable
 
-@pytest.mark.skipif(not test_gdcm_decoder, reason=gdcm_missing_message)
+
+@pytest.mark.skipif(not HAVE_GDCM, reason=gdcm_missing_message)
 class GDCM_JPEG2000Tests_with_gdcm(unittest.TestCase):
     def setUp(self):
         self.jpeg_2k = dcmread(jpeg2000_name)
@@ -288,11 +296,11 @@ class GDCM_JPEG2000Tests_with_gdcm(unittest.TestCase):
         self.sc_rgb_jpeg2k_gdcm_KY = dcmread(sc_rgb_jpeg2k_gdcm_KY)
         self.ground_truth_sc_rgb_jpeg2k_gdcm_KY_gdcm = dcmread(
             ground_truth_sc_rgb_jpeg2k_gdcm_KY_gdcm)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [numpy_handler, gdcm_handler]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
 
     def test_JPEG2000(self):
         """JPEG2000: Returns correct values for sample data elements"""
@@ -322,6 +330,8 @@ class GDCM_JPEG2000Tests_with_gdcm(unittest.TestCase):
             "Decoded pixel data is not all {0} "
             "(mean == {1})".format(b.mean(), a.mean()))
 
+        assert a.flags.writeable
+
     def test_emri_JPEG2000PixelArray(self):
         a = self.emri_jpeg_2k_lossless.pixel_array
         b = self.emri_small.pixel_array
@@ -331,6 +341,8 @@ class GDCM_JPEG2000Tests_with_gdcm(unittest.TestCase):
             "Decoded pixel data is not all {0} "
             "(mean == {1})".format(b.mean(), a.mean()))
 
+        assert a.flags.writeable
+
     def test_jpeg2000_lossy(self):
         a = self.sc_rgb_jpeg2k_gdcm_KY.pixel_array
         b = self.ground_truth_sc_rgb_jpeg2k_gdcm_KY_gdcm.pixel_array
@@ -343,18 +355,20 @@ class GDCM_JPEG2000Tests_with_gdcm(unittest.TestCase):
                 "Decoded pixel data is not all {0} "
                 "(mean == {1})".format(b.mean(), a.mean()))
 
+        assert a.flags.writeable
 
-@pytest.mark.skipif(not test_gdcm_decoder, reason=gdcm_missing_message)
+
+@pytest.mark.skipif(not HAVE_GDCM, reason=gdcm_missing_message)
 class GDCM_JPEGlossyTests_with_gdcm(unittest.TestCase):
 
     def setUp(self):
         self.jpeg_lossy = dcmread(jpeg_lossy_name)
         self.color_3d_jpeg = dcmread(color_3d_jpeg_baseline)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [numpy_handler, gdcm_handler]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
 
     def testJPEGlossless_odd_data_size(self):
         test_file = get_testdata_files('SC_rgb_small_odd_jpeg.dcm')[0]
@@ -380,13 +394,18 @@ class GDCM_JPEGlossyTests_with_gdcm(unittest.TestCase):
         self.assertEqual(a[420, 140], 244)
         self.assertEqual(a[230, 120], 95)
 
+        assert a.flags.writeable
+
     def test_JPEGBaselineColor3DPixelArray(self):
         self.assertEqual(
             self.color_3d_jpeg.PhotometricInterpretation,
             "YBR_FULL_422")
         a = self.color_3d_jpeg.pixel_array
+
+        assert a.flags.writeable
+
         self.assertEqual(a.shape, (120, 480, 640, 3))
-        a = self.color_3d_jpeg._convert_YBR_to_RGB(a)
+        a = _convert_YBR_FULL_to_RGB(a)
         # this test points were manually identified in Osirix viewer
         self.assertEqual(tuple(a[3, 159, 290, :]), (41, 41, 41))
         self.assertEqual(tuple(a[3, 169, 290, :]), (57, 57, 57))
@@ -397,10 +416,10 @@ class GDCM_JPEGlossyTests_with_gdcm(unittest.TestCase):
 
 @pytest.fixture(scope="module")
 def test_with_gdcm():
-    original_handlers = pydicom.config.image_handlers
-    pydicom.config.image_handlers = [numpy_handler, gdcm_handler]
+    original_handlers = pydicom.config.pixel_data_handlers
+    pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
     yield original_handlers
-    pydicom.config.image_handlers = original_handlers
+    pydicom.config.pixel_data_handlers = original_handlers
 
 
 if have_pytest_param:
@@ -549,7 +568,7 @@ else:
 
 
 @pytest.mark.skipif(
-    not test_gdcm_decoder,
+    not HAVE_GDCM,
     reason=gdcm_missing_message)
 @pytest.mark.parametrize(
     "image,PhotometricInterpretation,results,convert_yuv_to_rgb",
@@ -563,9 +582,12 @@ def test_PI_RGB(test_with_gdcm,
     t = dcmread(image)
     assert t.PhotometricInterpretation == PhotometricInterpretation
     a = t.pixel_array
+
+    assert a.flags.writeable
+
     assert a.shape == (100, 100, 3)
     if convert_yuv_to_rgb:
-        a = t._convert_YBR_to_RGB(a)
+        a = _convert_YBR_FULL_to_RGB(a)
     # this test points are from the ImageComments tag
     assert tuple(a[5, 50, :]) == results[0]
     assert tuple(a[15, 50, :]) == results[1]
@@ -580,15 +602,15 @@ def test_PI_RGB(test_with_gdcm,
     assert t.PhotometricInterpretation == PhotometricInterpretation
 
 
-@pytest.mark.skipif(not test_gdcm_decoder, reason=gdcm_missing_message)
+@pytest.mark.skipif(not HAVE_GDCM, reason=gdcm_missing_message)
 class GDCM_JPEGlosslessTests_with_gdcm(unittest.TestCase):
     def setUp(self):
         self.jpeg_lossless = dcmread(jpeg_lossless_name)
-        self.original_handlers = pydicom.config.image_handlers
-        pydicom.config.image_handlers = [numpy_handler, gdcm_handler]
+        self.original_handlers = pydicom.config.pixel_data_handlers
+        pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
 
     def tearDown(self):
-        pydicom.config.image_handlers = self.original_handlers
+        pydicom.config.pixel_data_handlers = self.original_handlers
 
     def testJPEGlossless(self):
         """JPEGlossless: Returns correct values for sample data elements"""
@@ -610,3 +632,5 @@ class GDCM_JPEGlosslessTests_with_gdcm(unittest.TestCase):
         # this test points were manually identified in Osirix viewer
         self.assertEqual(a[420, 140], 227)
         self.assertEqual(a[230, 120], 105)
+
+        assert a.flags.writeable
