@@ -1,9 +1,5 @@
-# valuerep.py
+# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
 """Special classes for DICOM value representations (VR)"""
-# Copyright (c) 2008-2012 Darcy Mason
-# This file is part of pydicom, released under a modified MIT license.
-#    See the file LICENSE included with this distribution, also
-#    available at https://github.com/pydicom/pydicom
 from copy import deepcopy
 from decimal import Decimal
 import re
@@ -28,17 +24,20 @@ extra_length_VRs = ('OB', 'OD', 'OF', 'OL', 'OW', 'SQ', 'UC', 'UN', 'UR', 'UT')
 # in (0008,0005) Specific Character Set
 # See PS-3.5 (2011), section 6.1.2 Graphic Characters
 # and PN, but it is handled separately.
-text_VRs = ('SH', 'LO', 'ST', 'LT', 'UC', 'UR', 'UT')
+text_VRs = ('SH', 'LO', 'ST', 'LT', 'UC', 'UT')
 
-match_string = b''.join([
-    b'(?P<single_byte>', br'(?P<family_name>[^=\^]*)',
-    br'\^?(?P<given_name>[^=\^]*)', br'\^?(?P<middle_name>[^=\^]*)',
-    br'\^?(?P<name_prefix>[^=\^]*)', br'\^?(?P<name_suffix>[^=\^]*)', b')',
-    b'=?(?P<ideographic>[^=]*)', b'=?(?P<phonetic>[^=]*)$'
-])
+# Delimiters for text strings and person name that reset the encoding.
+# See PS3.5, Section 6.1.2.5.3
+# Note: We use characters for Python 2 and character codes for Python 3
+# because these are the types yielded if iterating over a byte string.
 
-match_string_uni = re.compile(match_string.decode('iso8859'))
-match_string_bytes = re.compile(match_string)
+# Characters/Character codes for text VR delimiters: LF, CR, TAB, FF
+TEXT_VR_DELIMS = ({'\n', '\r', '\t', '\f'} if compat.in_py2
+                  else {0x0d, 0x0a, 0x09, 0x0c})
+
+# Character/Character code for PN delimiter: name part separator '^'
+# (the component separator '=' is handled separately)
+PN_DELIMS = {'^'} if compat.in_py2 else {0xe5}
 
 
 class DA(date):
@@ -58,10 +57,10 @@ class DA(date):
             setattr(self, slot, value)
 
     def __reduce__(self):
-        return super(DA, self).__reduce__() + (self.__getstate__(), )
+        return super(DA, self).__reduce__() + (self.__getstate__(),)
 
     def __reduce_ex__(self, protocol):
-        return super(DA, self).__reduce__() + (self.__getstate__(), )
+        return super(DA, self).__reduce__() + (self.__getstate__(),)
 
     def __new__(cls, val):
         """Create an instance of DA object.
@@ -111,6 +110,9 @@ class DA(date):
         else:
             return super(DA, self).__str__()
 
+    def __repr__(self):
+        return "\"" + str(self) + "\""
+
 
 class DT(datetime):
     """Store value for DICOM VR DT (DateTime) as datetime.datetime.
@@ -130,10 +132,10 @@ class DT(datetime):
             setattr(self, slot, value)
 
     def __reduce__(self):
-        return super(DT, self).__reduce__() + (self.__getstate__(), )
+        return super(DT, self).__reduce__() + (self.__getstate__(),)
 
     def __reduce_ex__(self, protocol):
-        return super(DT, self).__reduce__() + (self.__getstate__(), )
+        return super(DT, self).__reduce__() + (self.__getstate__(),)
 
     @staticmethod
     def _utc_offset(offset, name):
@@ -218,6 +220,9 @@ class DT(datetime):
         else:
             return super(DT, self).__str__()
 
+    def __repr__(self):
+        return "\"" + str(self) + "\""
+
 
 class TM(time):
     """Store value for DICOM VR of TM (Time) as datetime.time.
@@ -237,10 +242,10 @@ class TM(time):
             setattr(self, slot, value)
 
     def __reduce__(self):
-        return super(TM, self).__reduce__() + (self.__getstate__(), )
+        return super(TM, self).__reduce__() + (self.__getstate__(),)
 
     def __reduce_ex__(self, protocol):
-        return super(TM, self).__reduce__() + (self.__getstate__(), )
+        return super(TM, self).__reduce__() + (self.__getstate__(),)
 
     def __new__(cls, val):
         """Create an instance of TM object from a string.
@@ -298,6 +303,9 @@ class TM(time):
         else:
             return super(TM, self).__str__()
 
+    def __repr__(self):
+        return "\"" + str(self) + "\""
+
 
 class DSfloat(float):
     """Store values for DICOM VR of DS (Decimal String) as a float.
@@ -478,7 +486,7 @@ class IS(int):
         if isinstance(val, (float, Decimal)) and newval != val:
             raise TypeError("Could not convert value to integer without loss")
         # Checks in case underlying int is >32 bits, DICOM does not allow this
-        check_newval = (newval < -2**31 or newval >= 2**31)
+        check_newval = (newval < -2 ** 31 or newval >= 2 ** 31)
         if check_newval and config.enforce_valid_values:
             dcm_limit = "-2**31 to (2**31 - 1) for IS"
             message = "Value exceeds DICOM limits of %s" % (dcm_limit)
@@ -494,7 +502,7 @@ class IS(int):
 
     def __repr__(self):
         if hasattr(self, 'original_string'):
-            return "'" + self.original_string + "'"
+            return "\"" + self.original_string + "\""
         else:
             return "\"" + int.__str__(self) + "\""
 
@@ -521,41 +529,184 @@ def MultiString(val, valtype=str):
         return MultiValue(valtype, splitup)
 
 
+def _verify_encodings(encodings):
+    """Checks the encoding to ensure proper format"""
+    if encodings is not None and not isinstance(encodings, list):
+        return [encodings]
+
+    return encodings
+
+
+def _decode_personname(components, encodings):
+    """Return a list of decoded person name components.
+
+    Parameters
+    ----------
+    components : list of byte string
+        The list of the up to three encoded person name components
+    encodings : list of str
+        The Python encodings uses to decode `components`.
+
+    Returns
+    -------
+    text type
+        The unicode string representing the person name.
+        If the decoding of some component parts is not possible using the
+        given encodings, they are decoded with the first encoding using
+        replacement characters for bytes that cannot be decoded.
+    """
+    from pydicom.charset import decode_string
+
+    if isinstance(components[0], compat.text_type):
+        comps = components
+    else:
+        comps = [decode_string(comp, encodings, PN_DELIMS)
+                 for comp in components]
+    # Remove empty elements from the end to avoid trailing '='
+    while len(comps) and not comps[-1]:
+        comps.pop()
+    return comps
+
+
+def _encode_personname(components, encodings):
+    """Encode a list of text string person name components.
+
+    Parameters
+    ----------
+    components : list of text type
+        The list of the up to three unicode person name components
+    encodings : list of str
+        The Python encodings uses to encode `components`.
+
+    Returns
+    -------
+    byte string
+        The byte string that can be written as a PN DICOM tag value.
+        If the encoding of some component parts is not possible using the
+        given encodings, they are encoded with the first encoding using
+        replacement bytes for characters that cannot be encoded.
+    """
+    from pydicom.charset import encode_string
+
+    encoded_comps = []
+    for comp in components:
+        groups = [encode_string(group, encodings)
+                  for group in comp.split('^')]
+        encoded_comps.append(b'^'.join(groups))
+
+    # Remove empty elements from the end
+    while len(encoded_comps) and not encoded_comps[-1]:
+        encoded_comps.pop()
+    return b'='.join(encoded_comps)
+
+
 class PersonName3(object):
-    def __init__(self, val, encodings=default_encoding):
+    def __init__(self, val, encodings=None, original_string=None):
+        # handle None `val` as empty string
+        val = val or ''
         if isinstance(val, PersonName3):
             encodings = val.encodings
-            val = val.original_string
-
-        self.original_string = val
-
-        self.encodings = self._verify_encodings(encodings)
-        self.parse(val)
-
-    def parse(self, val):
-        if isinstance(val, bytes):
-            matchstr = match_string_bytes
+            self.original_string = val.original_string
+            self._components = str(val).split('=')
+        elif isinstance(val, bytes):
+            # this is the raw byte string - decode it on demand
+            self.original_string = val
+            self._components = None
         else:
-            matchstr = match_string_uni
+            # this is the decoded string - save the original string if
+            # available for easier writing back
+            self.original_string = original_string
+            self._components = val.split('=')
 
-        matchobj = re.match(matchstr, val)
+        self.encodings = _verify_encodings(encodings) or [default_encoding]
+        self._dict = {}
 
-        self.__dict__.update(matchobj.groupdict())
+    def _create_dict(self):
+        """Creates a dictionary of person name group and component names.
+        Used exclusively for `formatted` for backwards compatibility."""
+        if not self._dict:
+            for name in ('family_name', 'given_name', 'middle_name',
+                         'name_prefix', 'name_suffix',
+                         'ideographic', 'phonetic'):
+                self._dict[name] = getattr(self, name, '')
 
-        groups = matchobj.groups()
-        self.components = [groups[i] for i in (0, -2, -1)]
+    @property
+    def components(self):
+        """Return the up to three decoded person name components, representing
+        the alphabetic, ideographic and phonetic representations as a list
+        of unicode strings.
+        """
+        if self._components is None:
+            groups = self.original_string.split(b'=')
+            self._components = _decode_personname(groups, self.encodings)
+
+        return self._components
+
+    def _name_part(self, i):
+        try:
+            return self.components[0].split('^')[i]
+        except IndexError:
+            return ''
+
+    @property
+    def family_name(self):
+        """Return the first (family name) group of the alphabetic person name
+        representation as a unicode string"""
+        return self._name_part(0)
+
+    @property
+    def given_name(self):
+        """Return the second (given name) group of the alphabetic person name
+        representation as a unicode string"""
+        return self._name_part(1)
+
+    @property
+    def middle_name(self):
+        """Return the third (middle name) group of the alphabetic person name
+        representation as a unicode string"""
+        return self._name_part(2)
+
+    @property
+    def name_prefix(self):
+        """Return the fourth (name prefix) group of the alphabetic person name
+        representation as a unicode string"""
+        return self._name_part(3)
+
+    @property
+    def name_suffix(self):
+        """Return the fifth (name suffix) group of the alphabetic person name
+        representation as a unicode string"""
+        return self._name_part(4)
+
+    @property
+    def ideographic(self):
+        """Return the second (ideographic) person name component as a
+        unicode string"""
+        try:
+            return self.components[1]
+        except IndexError:
+            return ''
+
+    @property
+    def phonetic(self):
+        """Return the third (phonetic) person name component as a
+        unicode string"""
+        try:
+            return self.components[2]
+        except IndexError:
+            return ''
 
     def __eq__(self, other):
-        return self.original_string == other
+        return str(self) == other
 
     def __ne__(self, other):
         return not self == other
 
     def __str__(self):
-        return self.original_string.__str__()
+        return '='.join(self.components).__str__()
 
     def __repr__(self):
-        return self.original_string.__repr__()
+        return '='.join(self.components).__repr__()
 
     # For python 3, any override of __cmp__ or __eq__
     # immutable requires explicit redirect of hash
@@ -565,58 +716,63 @@ class PersonName3(object):
     __hash__ = object.__hash__
 
     def decode(self, encodings=None):
-        encodings = self._verify_encodings(encodings)
+        """Return the patient name decoded by the given encodings.
 
-        from pydicom.charset import clean_escseq
-        if not isinstance(self.components[0], bytes):
-            comps = self.components
-        else:
-            comps = [
-                clean_escseq(comp.decode(enc), encodings)
-                for comp, enc in zip(self.components, encodings)
-            ]
+        Parameters
+        ----------
+        encodings : list of str
+            The list of encodings used for decoding the byte string. If not
+            given, the initial encodings set in the object are used.
 
-        while len(comps) and not comps[-1]:
-            comps.pop()
-
-        return PersonName3('='.join(comps), encodings)
+        Returns
+        -------
+        PersonName3
+            A person name object that will return the decoded string with
+            the given encodings on demand. If the encodings are not given,
+            the current object is returned.
+        """
+        # in the common case (encoding did not change) we decode on demand
+        if encodings is None or encodings == self.encodings:
+            return self
+        # the encoding was unknown or incorrect - create a new
+        # PersonName object with the changed encoding
+        encodings = _verify_encodings(encodings)
+        return PersonName3(self.original_string, encodings)
 
     def encode(self, encodings=None):
-        encodings = self._verify_encodings(encodings)
+        """Return the patient name decoded by the given encodings.
 
-        if isinstance(self.components[0], bytes):
-            comps = self.components
-        else:
-            comps = [
-                C.encode(enc) for C, enc in zip(self.components, encodings)
-            ]
+        Parameters
+        ----------
+        encodings : list of str
+            The list of encodings used for encoding the unicode string. If
+            not given, the initial encodings set in the object are used.
 
-        # Remove empty elements from the end
-        while len(comps) and not comps[-1]:
-            comps.pop()
-
-        return b'='.join(comps)
+        Returns
+        -------
+        bytes
+            The person name encoded with the given encodings as a byte string.
+            If no encoding is given, the original byte string is returned, if
+            available, otherwise each group of the patient name is encoded
+            with the first matching of the given encodings.
+        """
+        encodings = _verify_encodings(encodings) or self.encodings
+        # if the encoding is not the original encoding, we have to return
+        # a re-encoded string (without updating the original string)
+        if encodings != self.encodings:
+            return _encode_personname(self.components, encodings)
+        if self.original_string is None:
+            # if the original encoding was not set, we set it now
+            self.original_string = _encode_personname(self.components,
+                                                      encodings)
+        return self.original_string
 
     def family_comma_given(self):
         return self.formatted('%(family_name)s, %(given_name)s')
 
     def formatted(self, format_str):
-        if isinstance(self.original_string, bytes):
-            return format_str % self.decode(default_encoding).__dict__
-        else:
-            return format_str % self.__dict__
-
-    def _verify_encodings(self, encodings):
-        if encodings is None:
-            return self.encodings
-
-        if not isinstance(encodings, list):
-            encodings = [encodings] * 3
-
-        if len(encodings) == 2:
-            encodings.append(encodings[1])
-
-        return encodings
+        self._create_dict()
+        return format_str % self._dict
 
 
 class PersonNameBase(object):
@@ -717,29 +873,13 @@ class PersonNameUnicode(PersonNameBase, compat.text_type):
                  from pydicom.charset.python_encodings mapping
                  of values in DICOM data element (0008,0005).
         """
-        # in here to avoid circular import
-        from pydicom.charset import clean_escseq
-
-        # Make the possible three character encodings explicit:
-        if not isinstance(encodings, list):
-            encodings = [encodings] * 3
-        if len(encodings) == 2:
-            encodings.append(encodings[1])
-        components = val.split(b"=")
-        # Remove the first encoding if only one component is present
-        if (len(components) == 1):
-            del encodings[0]
-
-        comps = [
-            clean_escseq(C.decode(enc), encodings)
-            for C, enc in zip(components, encodings)
-        ]
+        encodings = _verify_encodings(encodings)
+        comps = _decode_personname(val.split(b"="), encodings)
         new_val = u"=".join(comps)
-
         return compat.text_type.__new__(cls, new_val)
 
     def __init__(self, val, encodings):
-        self.encodings = self._verify_encodings(encodings)
+        self.encodings = _verify_encodings(encodings)
         PersonNameBase.__init__(self, val)
 
     def __copy__(self):
@@ -764,32 +904,10 @@ class PersonNameUnicode(PersonNameBase, compat.text_type):
             setattr(new_person, k, deepcopy(v, memo))
         return new_person
 
-    def _verify_encodings(self, encodings):
-        """Checks the encoding to ensure proper format"""
-        if encodings is None:
-            return self.encodings
-
-        if not isinstance(encodings, list):
-            encodings = [encodings] * 3
-
-        if len(encodings) == 2:
-            encodings.append(encodings[1])
-
-        return encodings
-
     def encode(self, encodings):
         """Encode the unicode using the specified encoding"""
-        encodings = self._verify_encodings(encodings)
-
-        components = self.split('=')
-
-        comps = [C.encode(enc) for C, enc in zip(components, encodings)]
-
-        # Remove empty elements from the end
-        while len(comps) and not comps[-1]:
-            comps.pop()
-
-        return '='.join(comps)
+        encodings = _verify_encodings(encodings) or self.encodings
+        return _encode_personname(self.split('='), encodings)
 
     def family_comma_given(self):
         """Return name as 'Family-name, Given-name'"""
